# Model Transformations

POMDPModelTools contains several tools for transforming problems into other classes so that they can be used by different solvers.

## Fully Observable POMDP

```@docs
FullyObservablePOMDP
```

## Generative Belief MDP

Every POMDP is an MDP on the belief space `GenerativeBeliefMDP` creates a generative model for that MDP.

<aside class="warning">
WARNING: The reward generated by the `GenerativeBeliefMDP` is the reward for a *single state sampled from the belief*; it is not the expected reward for that belief transition (though, in expectation, they are equivalent of course). Implementing the model with the expected reward requires a custom implementation because belief updaters do not typically deal with reward.
</aside>

```@docs
GenerativeBeliefMDP
```

### Example

```@meta
DocTestSetup = quote
    using Pkg
    Pkg.add("POMDPModels")
    Pkg.add("BeliefUpdaters")
    # the thing below should be a jldoctest once POMDPSimulators gets registered
end
```

```julia
using POMDPModels
using POMDPModelTools
using BeliefUpdaters

pomdp = BabyPOMDP()
updater = DiscreteUpdater(pomdp)

belief_mdp = GenerativeBeliefMDP(pomdp, updater)
@show statetype(belief_mdp) # POMDPModels.BoolDistribution

for (a, r, sp) in stepthrough(belief_mdp, RandomPolicy(belief_mdp), "a,r,sp", max_steps=5)
    @show a, r, sp
end
```

```@meta
DocTestSetup = nothing
```

## Underlying MDP

```@docs
UnderlyingMDP
```
